/*
  ==============================================================================

   This file is part of the Plugin Consultant - VST API Logger
   Copyright 2005 by Ofer Shem Tov.

  ------------------------------------------------------------------------------

   Plugin Consultant can be redistributed and/or modified under the terms of the
   GNU General Public License, as published by the Free Software Foundation;
   either version 2 of the License, or (at your option) any later version.

   Plugin Consultant is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Plugin Consultant; if not, visit www.gnu.org/licenses or write to the
   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
   Boston, MA 02111-1307 USA

  ==============================================================================
*/

#ifdef _MSC_VER
#pragma warning (disable : 4996)
#endif

#include <aeffectx.h>
#include <juce.h>
#include "PCLogger.h"
#include "PCWindow.h"

extern AEffect* loadVstPlugin(audioMasterCallback);
extern void	unloadPlugin();

// the proxy VST effect
AEffect proxyEffect;

// the original VST effect
AEffect* vstEffect = NULL;

// the plugin logger
PCLogger pcLogger;

// remember last directory used
File lastDir;

// the Logger GUI
static PCWindow *window = NULL;

// this DLL path name
File dllPath;

#if MAC
extern short bundleRes;
#endif // MAC

#if WIN32
#undef T 
#undef Rectangle
#define Array tempArray
#include <windows.h>
#undef Array
#define T(x) x 

// the PluginConsultant DLL instance
HINSTANCE hInstance;
#endif // WIN32


//------------------------------------------------------------------------
#if WIN32
void getModuleFileName()
{
	char name[1024];
	name[0] = 0;
	GetModuleFileName( hInstance, name, 1024 );
	dllPath = name;
}
#endif
//------------------------------------------------------------------------
#if MAC
void getModuleFileName()
{
	CFBundleRef myBundle = CFBundleGetBundleWithIdentifier(CFSTR("com.achitophelconsulting.PluginConsultant") );

	if ( myBundle )
	{
		CFRetain( myBundle );
		CFURLRef myBundleURL = CFBundleCopyBundleURL( myBundle );
		
		if ( myBundleURL )
		{
			char name[1024];
			name[0] = 0;
			CFStringRef tmpPath = CFURLCopyPath( myBundleURL );
			CFStringGetCString( tmpPath, name, 1024, kCFStringEncodingASCII);
			dllPath = name;
			CFRelease( tmpPath );
			CFRelease( myBundleURL );
		}

		CFRelease( myBundle );
	}
}
#endif

//------------------------------------------------------------------------
void startPluginConsultant()
{
	// start JUCE
	initialiseJuce_GUI();

	// set logger name to this DLL/Bundle name until it is changed later to plugin name
	getModuleFileName();
	pcLogger.init( dllPath.getFileName() ); ;
	pcLogger.log( "Plugin Consultant - " +
				dllPath.getFullPathName() );
	pcLogger.log( Time::getCurrentTime().toString( true, true ) );

	// load state from XML file with the same name as this DLL
	XmlDocument loggerDocument( dllPath.withFileExtension( "xml" ) );
	XmlElement* outerElement = loggerDocument.getDocumentElement();

	if ( outerElement != NULL )
	{
		// get saved lastDir
		XmlElement* lastDirElement = outerElement->getChildByName( T("LastDir") );

		if ( lastDirElement != NULL )
		{
			XmlElement* lastDirTextElement = lastDirElement->getChildElement( 0 );
			if ( lastDirTextElement != NULL )
				lastDir = lastDirTextElement->getText();
		}

		// find logger state within the document and load it
		XmlElement* loggerElement = outerElement->getChildByName( T("Logger") );

		if ( loggerElement != NULL )
			pcLogger.loadState( *loggerElement );

		delete outerElement;
	}
}

//------------------------------------------------------------------------
void stopPluginConsultant()
{
	// save lastdir
	XmlElement* lastDirElement = new XmlElement( "LastDir" );
	lastDirElement->addTextElement( lastDir.getFullPathName() );

	// save into logger element in the file, overriding what might be in this element
	XmlElement* loggerElement = new XmlElement( "Logger" );
	pcLogger.saveState( *loggerElement );

	XmlElement outerElement( "PluginConsultant" );
	outerElement.addChildElement( lastDirElement );
	outerElement.addChildElement( loggerElement );

	// save logger state to XML file with the same name as this DLL
	if ( !outerElement.writeToFile( dllPath.withFileExtension( "xml" ), String::empty ) )
		pcLogger.log( "Failed to write logger state to " + dllPath.withFileExtension( "xml" ).getFullPathName() );

	if ( window != NULL )
		deleteAndZero( window );
	unloadPlugin();

	// stop JUCE
	shutdownJuce_GUI();
}

//------------------------------------------------------------------------
long VSTCALLBACK proxyDispatcher(AEffect *effect, long opCode, long index, long value,void *ptr, float opt)
{
	pcLogger.logVstDispatcher(opCode,index,value,ptr,opt);
#ifdef MAC
	short res = 0;
	switch ( opCode )
	{
		case effEditOpen:
		case effEditDraw:
		case effEditMouse:
		case effEditKey:
		case effEditTop:
		case effEditKeyDown:
		case effEditKeyUp:
			res = CurResFile();
			UseResFile( bundleRes );
			break;
	}
#endif // MAC
	long result = vstEffect->dispatcher(vstEffect, opCode, index, value, ptr, opt);
#ifdef MAC
	if ( res != 0 )
		UseResFile( res );
#endif // MAC
	pcLogger.logVstDispatcherReturn(opCode, ptr, result);

	// stop PluginConsultant when getting effClose
	if ( opCode == effClose )
	{
		stopPluginConsultant();
		vstEffect = NULL;
	}

	return result;
}

//------------------------------------------------------------------------
void VSTCALLBACK proxyProcess(AEffect *effect, float **inputs, float **outputs, long sampleframes)
{
	pcLogger.logVstProcess(inputs,outputs,sampleframes,false);
	vstEffect->process(effect, inputs, outputs, sampleframes);
}

//------------------------------------------------------------------------
void VSTCALLBACK proxySetParameter(AEffect *effect, long index, float parameter)
{
	pcLogger.logVstSetParameter( index, parameter );
	vstEffect->setParameter(effect, index, parameter);
}

//------------------------------------------------------------------------
float VSTCALLBACK proxyGetParameter(AEffect *effect, long index)
{
	pcLogger.logVstGetParameter( index );
	float result = vstEffect->getParameter(effect, index);
	return result;
}

//------------------------------------------------------------------------
void VSTCALLBACK proxyProcessReplacing(AEffect *effect, float **inputs, float **outputs, long sampleframes)
{
	pcLogger.logVstProcess(inputs,outputs,sampleframes,true);
	vstEffect->processReplacing(effect, inputs, outputs, sampleframes);
}

//------------------------------------------------------------------------
void copyAEffect()
{
	// copy the loaded effect fields to our proxy effect
	proxyEffect = *vstEffect;

	// but set the all the effect functions to our functions
	proxyEffect.dispatcher = proxyDispatcher;
	proxyEffect.process = proxyProcess;
	proxyEffect.setParameter = proxySetParameter;
	proxyEffect.getParameter = proxyGetParameter;
	proxyEffect.processReplacing = proxyProcessReplacing;
}

//------------------------------------------------------------------------
#if MAC
#pragma export on
#endif

#if WIN32
__declspec(dllexport) void* main (audioMasterCallback audioMaster)
#else
extern "C" AEffect* main_macho (audioMasterCallback audioMaster)
#endif
{
	// if we are already logging some VST plugin then refuse to log another plugin
	if ( vstEffect != NULL )
	{
		pcLogger.log( T("Plugin Consultant is exiting VST main function with NULL since we are already hosting a plugin") );
		return NULL;
	}
	
	startPluginConsultant();


	// get the hosted plugin
	vstEffect = loadVstPlugin(audioMaster);

	// if failed then exit without nothing
	if ( vstEffect == NULL )
	{
		stopPluginConsultant();
		return NULL;
	}

	// copy the AEffect structure to our proxy effect but replace original functions with proxy functions
	copyAEffect();

	// create GUI
	window = new PCWindow(); 
	window->toFront (true);

	// start GUI updates
	window->startTimer(50);

	return &proxyEffect;
}

#if MAC
#pragma export off
#endif

#if WIN32
//------------------------------------------------------------------------
BOOL WINAPI DllMain (HINSTANCE hInst, DWORD dwReason, LPVOID lpvReserved)
{
	hInstance = hInst;

	if ( dwReason == DLL_PROCESS_ATTACH )
	{
		PlatformUtilities::setCurrentModuleInstanceHandle (hInst);
		pcLogger.log( T("DLL_PROCESS_ATTACH") );
	}
	else if ( dwReason == DLL_PROCESS_DETACH )
		pcLogger.log( T("DLL_PROCESS_DETACH") );
	else if ( dwReason == DLL_THREAD_ATTACH )
		pcLogger.log( T("DLL_THREAD_ATTACH") );
	else if ( dwReason == DLL_THREAD_DETACH )
		pcLogger.log( T("DLL_THREAD_DETACH") );
	else
		pcLogger.log( T("DllMain") );

	return 1;
}
#endif // WIN32
