/*
  ==============================================================================

   This file is part of the Plugin Consultant - VST API Logger
   Copyright 2005 by Ofer Shem Tov.

  ------------------------------------------------------------------------------

   Plugin Consultant can be redistributed and/or modified under the terms of the
   GNU General Public License, as published by the Free Software Foundation;
   either version 2 of the License, or (at your option) any later version.

   Plugin Consultant is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Plugin Consultant; if not, visit www.gnu.org/licenses or write to the
   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
   Boston, MA 02111-1307 USA

  ==============================================================================
*/

#ifdef _MSC_VER
#pragma warning (disable : 4996)
#endif

#include <aeffectx.h>
#include <juce.h>
#include "juce1.8_TabbedComponent.h"
#include "PCLogger.h"
#include "ACLogoData.h"
#include "ACDonateButtonData.h"
#include "PCWindow.h"

extern PCLogger pcLogger;
extern AEffect proxyEffect;

#define PC_ENABLE_ALL_BUTTON		T("Enable All")
#define PC_DISABLE_ALL_BUTTON		T("Disable All")
#define PC_INVERSE_ALL_BUTTON		T("Inverse All")
#define PC_RESET_COUNTERS_BUTTON	T("Reset Counters")
#define PC_START_LOG_BUTTON			T("Start Logging")
#define PC_STOP_LOG_BUTTON			T("Stop Logging")
#define PC_STOP_LOG_TOOLTIP			T("Stop logging")
#define PC_START_LOG_TOOLTIP		T("Start logging")
#define PC_TIME_BUTTON				T("TimeButton")
#define PC_TIME_TOOLTIP				T("Click to reset time display and log a time reset event")
#define PC_HELP_BUTTON				T("Help...")
#define PC_HELP_TOOLTIP				T("View Plugin Consultant user guide on the Web")

// pixel values
enum {

	pcWindowW = 774,
	pcWindowH = 666,
	pcOpCodeHeight = 16,
	pcOpCodeCounterWidth = 58,
	pcOpCodeCounterSpace = 4,
	pcOpCodeLedSize = 10,
	pcTabWidth = 200,
	pcButtonXSpace = 10,
	pcButtonWidth = pcTabWidth - (pcButtonXSpace * 2),
	pcButtonTopSpace = 10,
	pcButtonYSpace = 8,
	pcButtonHeight = 25,
	pcOpCodeWidth = 280,
	pcTabbedSpace = 2,
	pcOpCodeFontHeight = 15,
	pcEffectInfoYSpace = 5,
	pcEffectInfoXSpace = 0,
	pcLogoWidth = 181,
	pcLogoHeight = 136,
	pcDonateButtonWidth = 72,
	pcDonateButtonHeight = 29,
};

// colour values, cannot be initialized here since JUCE Colours are also static and might be initialized yet
static Colour pcButtonBackColor = Colour( 206, 219, 227 );
static Colour pcButtonTextColor = Colour( 0, 0, 0 );
static Colour pcButtonOutColor = Colour( 37, 100, 139 );

static Colour pcOpCodeTextColorZero = Colour( 139, 139, 139 );
static Colour pcOpCodeTextColorNonZero = Colour ( 0, 0, 0 );
static Colour pcOpCodeBackColor = Colour( 242, 242, 242 );
static Colour pcOpCodeOverColor = Colour( 229, 226, 218 );
static Colour pcOpCodeOutColor = Colour( 224, 224, 224 );
static Colour pcOpCodeLedOnColor = Colour( 229, 183, 128 );
static Colour pcOpCodeLedOffColor = Colour( 253, 241, 212 );
static Colour pcOpCodeLedOutColor = Colour( 243, 219, 176 );

static Colour pcAEffectTextColor = Colour ( 77, 77, 77 );
static Colour pcAEffectBackColor = pcOpCodeBackColor;			

static Colour pcTitleButtonNormalColor = Colour( 206, 219, 227 );
static Colour pcTitleButtonOverColor = Colour( 176, 189, 197 );
static Colour pcTitleButtonDownColor = Colour( 37, 100, 139 );

static Colour pcTabFillColour = pcOpCodeBackColor;
static Colour pcTabTextColour = Colour ( 77, 77, 77 );
static Colour pcTabSelectedOutlineColour = Colour( 37, 100, 139  );
static Colour pcTabNormalOutlineColour = Colour( 206, 219, 227 );
static Colour pcTabDeselectedTabColour = Colour( 232, 232, 232 );

static Colour pcAboutTextColor = Colour ( 77, 77, 77 );
static Colour pcAboutBackColor = pcTabFillColour;

static Colour pcBackColor = Colour( 103, 147, 174 );

//-----------------------------------------------------------------------------------------
class PCVstOpCodeComponent : public Component {

public:

	// create an opcode component with given opcode name width
	PCVstOpCodeComponent( PCLogger::vstOpCodeGroup group_, long opCode_ );
	~PCVstOpCodeComponent();

	// init component after it has been sized
	void init();

	// update display according to function dynamic debug info
	void updateGUI();

	// toggle enable state
	void mouseDown( const MouseEvent &e );

	// paint according to logger and mouse status
	void paint( Graphics &g );

	// set enabled state in logger and GUI
	void setEnabledState( bool state );

	// toggle enabled state in logger and GUI
	void toggleEnabledState();

private:

	// function and op code we are showing
	PCLogger::vstOpCodeGroup group;
	long opCode;

	// counter as displayed in the counter label
	int count;
};

//-----------------------------------------------------------------------------------------
class PCVstOpCodeListComponent : public Component {

public:

	PCVstOpCodeListComponent( PCLogger::vstOpCodeGroup group_);
	~PCVstOpCodeListComponent();

	// init component after it has been sized
	virtual	void init();

	// update GUI according to logger status
	virtual void updateGUI();

	// set logging state of all opcodes
	void setOpCodesEnableState( bool state );

	// toggle logging state of all opcodes
	void toggleOpCodesEnableState();

protected:

	bool inited;

	PCLogger::vstOpCodeGroup group;

	Array<PCVstOpCodeComponent*> opCodeComponents;
};

//-----------------------------------------------------------------------------------------
class PCVstAEffectComponent : public PCVstOpCodeListComponent {

public:

	PCVstAEffectComponent();
	~PCVstAEffectComponent();

	// init the effect info
	virtual	void init();

	// see if need to repaint effect info
	virtual void updateGUI();

private:

	// copy of the proxy effect info
	AEffect effect;

	// for displaying AEffect structure
	TextEditor* effectInfo;

	// fill the effect info into the text editor
	void updateEffectInfo();

	// convert long to 4 chars string
	const String long2Chars( long );
};

//-----------------------------------------------------------------------------------------
class PCAboutContent : public Component {

public:

	// create about content
	PCAboutContent();
	~PCAboutContent();
};

//-----------------------------------------------------------------------------------------
class PCContentComponent : public OldTabbedComponent, public ButtonListener {

public:

	enum {
		pcFuncTab,
		pcEffTab,
		pcAudioMasterTab,
		pcAboutTab,
	};

	PCContentComponent();
	~PCContentComponent();

	// init component after it has been sized
	void init();

	// update GUI according to logger status
	void updateGUI();

	void 	tabWasClicked (int newSelectedIndex, bool clickedTabWasAlreadySelected,
							const ModifierKeys &currentModifiers);

	void 	buttonClicked (Button *button);

private:

	// tab content components
	PCVstOpCodeListComponent* opCodeListsComp[3];
	
	// about content
	PCAboutContent* aboutComp;

	// buttons
	TextButton* enableAllButton;
	TextButton* disableAllButton;
	TextButton* inverseAllButton;
	TextButton* startStopLogButton;
	TextButton* resetCountersButton;
	TextButton* timeButton;
	TextButton* helpButton;
	Time startTime;
	ToggleButton* logParamsButton;
	ToggleButton* logRetParamsButton;
	ToggleButton* logTimeButton;
	ImageButton* logoButton;
	ImageButton* donateButton;

	TooltipWindow	tooltipWindow;
};

//-----------------------------------------------------------------------------------------
PCContentComponent::PCContentComponent() : OldTabbedComponent( "PCContentComponent" ), startTime( Time::getCurrentTime() )
{
	setColours ( pcTabFillColour, pcTabTextColour, pcTabSelectedOutlineColour, pcTabNormalOutlineColour, pcTabDeselectedTabColour);

	// create content components
	opCodeListsComp[PCLogger::vstFunctions] = new PCVstAEffectComponent();
	opCodeListsComp[PCLogger::vstDispatcherOpCodes] = new PCVstOpCodeListComponent( PCLogger::vstDispatcherOpCodes );
	opCodeListsComp[PCLogger::vstAudioMasterOpCodes] = new PCVstOpCodeListComponent( PCLogger::vstAudioMasterOpCodes );
	
	aboutComp = new PCAboutContent();
}

//-----------------------------------------------------------------------------------------
PCContentComponent::~PCContentComponent()
{
	deleteAndZero(opCodeListsComp[PCLogger::vstFunctions]);
	deleteAndZero(opCodeListsComp[PCLogger::vstDispatcherOpCodes]);
	deleteAndZero(opCodeListsComp[PCLogger::vstAudioMasterOpCodes]);
	deleteAndZero(aboutComp);
	deleteAllChildren();
}

//-----------------------------------------------------------------------------------------
void PCContentComponent::init()
{
	// add tabs
	addTab( pcFuncTab, new OldTabbedComponentBasicTab( "AEffect" ) );
	addTab( pcEffTab, new OldTabbedComponentBasicTab( "Dispatcher OpCodes" ) );
	addTab( pcAudioMasterTab, new OldTabbedComponentBasicTab( "AudioMaster OpCodes" ) );
	addTab( pcAboutTab, new OldTabbedComponentBasicTab( "About" ) );
	setTabOrientation (OldTabbedComponent::TabsAtLeft, pcTabWidth, pcTabbedSpace, pcTabbedSpace );
	setCurrentlySelectedTab( 0 );

	// add buttons bellow bottom tab
	OldTabbedComponentBasicTab* bottomTab = getTabComponent( getNumTabs() - 1 );
	int y = bottomTab->getY() + bottomTab->getHeight() + pcButtonTopSpace;

	enableAllButton = new TextButton( PC_ENABLE_ALL_BUTTON, "Enable logging of all functions and OpCodes" );
	enableAllButton->setTextColour( pcButtonTextColor );
	enableAllButton->setBackgroundColours( pcButtonBackColor, pcButtonOutColor );
	enableAllButton->setBounds( pcButtonXSpace, y, pcButtonWidth, pcButtonHeight );
	addAndMakeVisible( enableAllButton );
	enableAllButton->addButtonListener( this );

	y = y + pcButtonYSpace + pcButtonHeight;
	disableAllButton = new TextButton( PC_DISABLE_ALL_BUTTON, "Disable logging of all functions and OpCodes" );
	disableAllButton->setTextColour( pcButtonTextColor );
	disableAllButton->setBackgroundColours( pcButtonBackColor, pcButtonOutColor );
	disableAllButton->setBounds( pcButtonXSpace, y, pcButtonWidth, pcButtonHeight );
	addAndMakeVisible( disableAllButton );
	disableAllButton->addButtonListener( this );

	y = y + pcButtonYSpace + pcButtonHeight;
	inverseAllButton = new TextButton( PC_INVERSE_ALL_BUTTON, "Inverse logging enable/disable state of all functions and OpCodes" );
	inverseAllButton->setTextColour( pcButtonTextColor );
	inverseAllButton->setBackgroundColours( pcButtonBackColor, pcButtonOutColor );
	inverseAllButton->setBounds( pcButtonXSpace, y, pcButtonWidth, pcButtonHeight );
	addAndMakeVisible( inverseAllButton );
	inverseAllButton->addButtonListener( this );

	y = y + pcButtonYSpace + pcButtonHeight;
	resetCountersButton = new TextButton( PC_RESET_COUNTERS_BUTTON, "Reset the counters of all functions and OpCodes" );
	resetCountersButton->setTextColour( pcButtonTextColor );
	resetCountersButton->setBackgroundColours( pcButtonBackColor, pcButtonOutColor );
	resetCountersButton->setBounds( pcButtonXSpace, y, pcButtonWidth, pcButtonHeight );
	addAndMakeVisible( resetCountersButton );
	resetCountersButton->addButtonListener( this );

	y = y + pcButtonYSpace + pcButtonHeight;
	startStopLogButton = new TextButton( PC_STOP_LOG_BUTTON, PC_STOP_LOG_TOOLTIP );
	startStopLogButton->setTextColour( pcButtonTextColor );
	startStopLogButton->setBackgroundColours( pcButtonBackColor, pcButtonOutColor );
	startStopLogButton->setBounds( pcButtonXSpace, y, pcButtonWidth, pcButtonHeight );
	addAndMakeVisible( startStopLogButton );
	startStopLogButton->addButtonListener( this );

	y = y + pcButtonYSpace + pcButtonHeight;
	timeButton = new TextButton( String::empty, PC_TIME_TOOLTIP );
	timeButton->setName( PC_TIME_BUTTON );
	timeButton->setTextColour( pcButtonTextColor );
	timeButton->setBackgroundColours( pcButtonBackColor, pcButtonOutColor );
	timeButton->setBounds( pcButtonXSpace, y, pcButtonWidth, pcButtonHeight );
	addAndMakeVisible( timeButton );
	timeButton->addButtonListener( this );

	y = y + pcButtonYSpace + pcButtonHeight;
	helpButton = new TextButton( PC_HELP_BUTTON, PC_HELP_TOOLTIP );
	helpButton->setTextColour( pcButtonTextColor );
	helpButton->setBackgroundColours( pcButtonBackColor, pcButtonOutColor );
	helpButton->setBounds( pcButtonXSpace, y, pcButtonWidth, pcButtonHeight );
	addAndMakeVisible( helpButton );
	helpButton->addButtonListener( this );

	// add toggle buttons
	y = y + pcButtonYSpace + pcButtonHeight;
	logParamsButton = new ToggleButton( "Log Parameters" );
	logParamsButton->setBounds( pcButtonXSpace, y, 0, pcButtonHeight );
	logParamsButton->setTooltip( "Enable/Disable logging of function parameters and return parameters" );
	logParamsButton->setToggleState( pcLogger.isLogParameters(), false );
	logParamsButton->changeWidthToFitText();
	addAndMakeVisible( logParamsButton );
	logParamsButton->addButtonListener( this );

	y = y + pcButtonYSpace + pcButtonHeight;
	logRetParamsButton = new ToggleButton( "Log Return Parameters" );
	logRetParamsButton->setBounds( pcButtonXSpace, y, 0, pcButtonHeight );
	logRetParamsButton->setTooltip( "Enable/Disable logging of function return parameters" );
	logRetParamsButton->setToggleState( pcLogger.isLogReturnParameters(), false );
	logRetParamsButton->changeWidthToFitText();
	addAndMakeVisible( logRetParamsButton );
	logRetParamsButton->addButtonListener( this );
	
	y = y + pcButtonYSpace + pcButtonHeight;
	logTimeButton = new ToggleButton( "Log Time" );
	logTimeButton->setBounds( pcButtonXSpace, y, 0, pcButtonHeight );
	logTimeButton->setTooltip( "Enable/Disable logging of the function call time" );
	logTimeButton->setToggleState( pcLogger.isLogTime(), false );
	logTimeButton->changeWidthToFitText();
	addAndMakeVisible( logTimeButton );
	logTimeButton->addButtonListener( this );

	// add logo button
	y = y + pcButtonYSpace + pcButtonHeight;
	logoButton = new ImageButton ( "ACLogoButton" );
	addAndMakeVisible( logoButton );
	Image* normalImage = ImageCache::getFromMemory(ACLogoData::aclogo_png, ACLogoData::aclogo_pngSize);
	Image* overImage = ImageCache::getFromMemory(ACLogoData::aclogo_png, ACLogoData::aclogo_pngSize);
	Image* downImage = ImageCache::getFromMemory(ACLogoData::aclogo_png, ACLogoData::aclogo_pngSize);
	logoButton->setImages (true, true, true,
							normalImage, 1.0f, Colours::transparentBlack,
							overImage, 0.75f, Colours::transparentBlack,
							downImage, 0.5f, Colours::transparentBlack,
							0.5f);

	logoButton->setBounds(pcButtonXSpace, y, pcLogoWidth, pcLogoHeight );
	logoButton->setTooltip ( "Sometimes, do the wrong thing" );
	logoButton->addButtonListener( this );

	// add donation button
	y = y + pcButtonYSpace / 2 + pcLogoHeight;
	donateButton = new ImageButton ( "ACDonateButton" );
	addAndMakeVisible( donateButton );
	Image* donateNormalImage = ImageCache::getFromMemory(ACDonateButtonData::donate_png, ACDonateButtonData::donate_pngSize);
	Image* donateOverImage = ImageCache::getFromMemory(ACDonateButtonData::donate_png, ACDonateButtonData::donate_pngSize);
	Image* donateDownImage = ImageCache::getFromMemory(ACDonateButtonData::donate_png, ACDonateButtonData::donate_pngSize);
	donateButton->setImages (true, true, true,
							donateNormalImage, 0.5f, Colours::transparentBlack,
							donateOverImage, 0.75f, Colours::transparentBlack,
							donateDownImage, 1.0f, Colours::transparentBlack,
							0.5f);

	donateButton->setBounds( pcButtonXSpace + (pcButtonWidth - pcDonateButtonWidth) / 2, y,
							pcDonateButtonWidth, pcDonateButtonHeight );
	donateButton->setTooltip ( "Do the right thing!" );
	donateButton->addButtonListener( this );
}

//-----------------------------------------------------------------------------------------
void PCContentComponent::updateGUI()
{
	// update GUI of visible component only, to save on CPU...
	if ( getCurrentlySelectedTab() != pcAboutTab )
		opCodeListsComp[getCurrentlySelectedTab()]->updateGUI();

	// update time display
	Time timeNow( Time::getCurrentTime() );
	timeButton->setButtonText( (timeNow - startTime).getDescription() );
}

//-----------------------------------------------------------------------------------------
void PCContentComponent::tabWasClicked (int newSelectedIndex, bool clickedTabWasAlreadySelected,
										const ModifierKeys &currentModifiers)
{
	if ( clickedTabWasAlreadySelected )
		return;
		
	if ( newSelectedIndex == pcAboutTab )
		setContentComponent( aboutComp );
	else
	{
		setContentComponent(opCodeListsComp[newSelectedIndex]);
		opCodeListsComp[newSelectedIndex]->init();
	}
}

//-----------------------------------------------------------------------------------------
void PCContentComponent::buttonClicked (Button *button)
{
	if ( button == enableAllButton )
	{
		pcLogger.setVstOpCodesEnabled( true );
		opCodeListsComp[getCurrentlySelectedTab()]->repaint();
	}
	else if ( button == disableAllButton )
	{
		pcLogger.setVstOpCodesEnabled( false );
		opCodeListsComp[getCurrentlySelectedTab()]->repaint();
	}
	else if ( button == inverseAllButton )
	{
		pcLogger.toggleVstOpCodesEnabled();
		opCodeListsComp[getCurrentlySelectedTab()]->repaint();
	}
	else if (  button == resetCountersButton )
	{
		pcLogger.resetVstOpCodeCounters();
		opCodeListsComp[getCurrentlySelectedTab()]->repaint();
	}
	else if ( button == startStopLogButton )
	{
		if ( !pcLogger.isOn() )
		{
			pcLogger.start();
			startStopLogButton->setTooltip( PC_STOP_LOG_TOOLTIP );
			startStopLogButton->setButtonText( PC_STOP_LOG_BUTTON );
			startStopLogButton->setName( PC_STOP_LOG_BUTTON );
		}
		else
		{
			pcLogger.stop();
			startStopLogButton->setTooltip( PC_START_LOG_TOOLTIP );
			startStopLogButton->setButtonText( PC_START_LOG_BUTTON );
			startStopLogButton->setName( PC_START_LOG_BUTTON );
		}
	}
	else if (  button == timeButton )
	{
		Time timeNow( Time::getCurrentTime() );
		pcLogger.log( "PluginConsultant timer reset after " + (timeNow - startTime).getDescription() );
		startTime = timeNow;
	}
	else if (  button == helpButton )
	{
		URL url( "http://www.achitophelconsulting.com/pluginconsultant/guide" );
		url.launchInDefaultBrowser();
	}
	else if ( button == logParamsButton )
	{
		pcLogger.setLogParameters( logParamsButton->getToggleState() );
	}
	else if ( button == logRetParamsButton )
	{
		pcLogger.setLogReturnParameters( logRetParamsButton->getToggleState() );
	}
	else if ( button == logTimeButton )
	{
		pcLogger.setLogTime( logTimeButton->getToggleState() );
	}
	else if ( button == logoButton )
	{
		URL url ( "http://www.achitophelconsulting.com" );
		url.launchInDefaultBrowser();
	}
	else if ( button == donateButton )
	{
		URL url( "http://www.achitophelconsulting.com/pluginconsultant/donate" );
		url.launchInDefaultBrowser();
	}
}

//-----------------------------------------------------------------------------------------
PCAboutContent::PCAboutContent()
{
	// pluginconsultant URL
	HyperlinkButton* urlButton = new HyperlinkButton( "www.achitophelconsulting.com/pluginconsultant",
													URL( "http://www.achitophelconsulting.com/pluginconsultant" ) );
	urlButton->setBounds( 5 , 140, 100, 18 );
	addAndMakeVisible( urlButton );
	urlButton->changeWidthToFitText();

	// copyrights
	TextEditor* copyright = new TextEditor( String::empty );
	copyright->setReadOnly( true );
	copyright->setMultiLine( true );
	copyright->setPopupMenuEnabled( true );
	copyright->setColours( pcAboutTextColor, pcAboutBackColor );
	copyright->setBounds( 5, 5, 270, 120 );
	copyright->setText( "Plugin Consultant 1.1\nCopyright 2005 by Ofer Shem Tov\n\nBuilt with the JUCE library\nwww.rawmaterialsoftware.com/juce\n\nVST Plugin Technology by Steinberg" );

	addAndMakeVisible( copyright );
}

//-----------------------------------------------------------------------------------------
PCAboutContent::~PCAboutContent()
{
	deleteAllChildren();
}

//-----------------------------------------------------------------------------------------
PCVstOpCodeListComponent::PCVstOpCodeListComponent( PCLogger::vstOpCodeGroup group_ )
{
	setName( "PCVstOpCodeListComponent " + String( group_ ) );
	group = group_;
	inited = false;
}

//-----------------------------------------------------------------------------------------
PCVstOpCodeListComponent::~PCVstOpCodeListComponent()
{
	deleteAllChildren();
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeListComponent::init()
{
	if ( inited )
		return;

	inited = true;

	int opCodes = pcLogger.getVstNumOpCodes( group );

	int row = 0, col = 0;

	// move to next column if filled all height with components
	for ( int i = 0; i < opCodes; i ++ )
	{
		if ( row + pcOpCodeHeight > getHeight() )
		{
			col = col + pcOpCodeWidth + 2;
			row = 0;
		}

		PCVstOpCodeComponent* opCodeComp = new PCVstOpCodeComponent( group, i );
		opCodeComp->setBounds( col, row, pcOpCodeWidth, pcOpCodeHeight );
		addAndMakeVisible( opCodeComp );
		opCodeComponents.add( opCodeComp );
		row = row + pcOpCodeHeight;
	}
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeListComponent::updateGUI()
{
	int opCodes = pcLogger.getVstNumOpCodes( group );

	for ( int i = 0; i < opCodes; i ++ )
		opCodeComponents[i]->updateGUI();
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeListComponent::setOpCodesEnableState( bool state )
{
	int opCodes = pcLogger.getVstNumOpCodes( group );

	for ( int i = 0; i < opCodes; i ++ )
		opCodeComponents[i]->setEnabledState( state );
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeListComponent::toggleOpCodesEnableState()
{
	int opCodes = pcLogger.getVstNumOpCodes( group );

	for ( int i = 0; i < opCodes; i ++ )
		opCodeComponents[i]->toggleEnabledState();
}

//-----------------------------------------------------------------------------------------
PCVstAEffectComponent::PCVstAEffectComponent() : PCVstOpCodeListComponent( PCLogger::vstFunctions )
{
	// clear the effect so it will be later updated correctly to match the proxy effect
	memset( &effect, 0, sizeof(AEffect) );
}

//-----------------------------------------------------------------------------------------
PCVstAEffectComponent::~PCVstAEffectComponent()
{
	deleteAllChildren();
}

//-----------------------------------------------------------------------------------------
void PCVstAEffectComponent::init()
{
	if ( inited )
		return;

	// in order that everything is sized before we start
	PCVstOpCodeListComponent::init();

	// put effect info under the last op code component
	effectInfo = new TextEditor( "AEffectTextEditor" );
	int y = opCodeComponents[opCodeComponents.size()-1]->getBottom();
	effectInfo->setBounds( 0, y + pcEffectInfoYSpace,
							getWidth() - pcEffectInfoXSpace * 2,
							getHeight() - y - pcEffectInfoYSpace * 2 );

	effectInfo->setReadOnly( true );
	effectInfo->setMultiLine( true );
	effectInfo->setPopupMenuEnabled( true );
	effectInfo->setColours( pcAEffectTextColor, pcAEffectBackColor );

	addAndMakeVisible( effectInfo );
}

//-----------------------------------------------------------------------------------------
void PCVstAEffectComponent::updateGUI()
{
	// update op codes GUI
	PCVstOpCodeListComponent::updateGUI();

	// repaint effect info if the AEffect structure changed
	if ( memcmp( &proxyEffect, &effect, sizeof(AEffect) ) != 0 )
	{
		effect = proxyEffect;
		updateEffectInfo();
	}
}

//-----------------------------------------------------------------------------------------
void PCVstAEffectComponent::updateEffectInfo()
{
	struct effFlagsInfo {

		long flagBit;
		char* flagName;
	};

	static const int numEffFlags = 10;
	static effFlagsInfo flagInfos[numEffFlags] = {
		{ effFlagsHasEditor, "effFlagsHasEditor" },
		{ effFlagsHasClip, "effFlagsHasClip" },
		{ effFlagsHasVu, "effFlagsHasVu" },
		{ effFlagsCanMono, "effFlagsCanMono" },
		{ effFlagsCanReplacing, "effFlagsCanReplacing" },
		{ effFlagsProgramChunks, "effFlagsProgramChunks" },
		{ effFlagsIsSynth, "effFlagsIsSynth" },
		{ effFlagsNoSoundInStop, "effFlagsNoSoundInStop" },
		{ effFlagsExtIsAsync, "effFlagsExtIsAsync" },
		{ effFlagsExtHasBuffer, "effFlagsExtHasBuffer" }
	};

	effectInfo->clear();
	
	String info;
	info = "magic: " + long2Chars( effect.magic );
	info = info + "\nnumPrograms: " + String( (int) effect.numPrograms );
	info = info + "\nnumParams: " + String( (int) effect.numParams );
	info = info + "\nnumInputs: " + String( (int) effect.numInputs );
	info = info + "\nnumOutputs: " + String( (int) effect.numOutputs );
	info = info + "\nflags: 0x" + String::toHexString( (int) effect.flags );

	for ( int i = 0; i < numEffFlags; i++ )
		if ( effect.flags & flagInfos[i].flagBit )
			info = info + "\n\t\t\t\t" + flagInfos[i].flagName;

	info = info + "\nresvd1: " + String( (int) effect.resvd1 );
	info = info + "\nresvd2: " + String( (int) effect.resvd2 );
	info = info + "\ninitialDelay: " + String( (int) effect.initialDelay );
	info = info + "\nrealQualities: " + String( (int) effect.realQualities );
	info = info + "\noffQualities: " + String( (int) effect.offQualities );
	info = info + "\nioRatio: " + String( effect.ioRatio );
	info = info + "\nobject: 0x" + String::toHexString( (int64) effect.object );
	info = info + "\nuser: 0x" + String::toHexString( (int64) effect.user );
	info = info + "\nuniqueID: " + long2Chars( effect.uniqueID );
	info = info + "\nversion: " + String( (int) effect.version );

	effectInfo->setText( info, false );
}

//-----------------------------------------------------------------------------------------
const String PCVstAEffectComponent::long2Chars( long v )
{
	// for Mac if needed
	uint32 l = swapIfBigEndian ( (uint32)v );

	// code from VSTHost http://www.hermannseib.com/
	char c[5];
	int i;
	for (i = 0; i < 4; i++)
	{
		c[i] = ((char *)&l)[3 - i];
		if (!c[i])
			c[i] = ' ';
	}
	c[i] = '\0';
	return c;
}

//-----------------------------------------------------------------------------------------
PCVstOpCodeComponent::PCVstOpCodeComponent( PCLogger::vstOpCodeGroup group_, long opCode_ )
{
	group = group_;
	opCode = opCode_;

	// set name so we can identify components of this type later on
	setName( "PCVstOpCodeComponent" );

	// we want to highlight when mouse if over an opcode
	setRepaintsOnMouseActivity( true );
}

//-----------------------------------------------------------------------------------------
PCVstOpCodeComponent::~PCVstOpCodeComponent()
{
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeComponent::updateGUI()
{
	long newCount = pcLogger.getVstOpCodeCount( group, opCode );

	// update count area only if count changed or we are forced to update
	if ( newCount != count )
	{
		// paint it all if count changed from 0
		if ( count == 0 )
			repaint();
		// or only the counter part if was not 0
		else
			repaint( pcOpCodeWidth - pcOpCodeCounterWidth, 0, pcOpCodeCounterWidth, pcOpCodeHeight );

		count = newCount;
	}
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeComponent::paint( Graphics &g )
{
	Colour textColor;

	// colors depends on count
	count = pcLogger.getVstOpCodeCount( group, opCode );

	if ( isMouseOver() )
		g.setColour( pcOpCodeOverColor );
	else
		g.setColour( pcOpCodeBackColor );
	g.fillRect( 1, 1, getWidth() - 2, getHeight() - 2 );

	g.setColour( pcOpCodeOutColor );
	g.drawLine( 0, (float) getHeight() - 1, (float) getWidth() - 1, (float) getHeight() - 1 );

	Colour ledColor;

	// led on/off according to enabled state
	if ( pcLogger.isVstOpCodeEnabled( group, opCode ) )
		ledColor = pcOpCodeLedOnColor;
	else
		ledColor = pcOpCodeLedOffColor;

	g.setColour( ledColor );
	g.fillEllipse( 3, 4, pcOpCodeLedSize - 2, pcOpCodeLedSize - 2 );
	g.setColour( pcOpCodeLedOutColor );
	g.drawEllipse( 2, 3, pcOpCodeLedSize, pcOpCodeLedSize, 1 );

	// opcode name
	if ( count )
		textColor = pcOpCodeTextColorNonZero;
	else
		textColor = pcOpCodeTextColorZero;

	g.setColour( textColor );
	g.setFont( pcOpCodeFontHeight );
	g.drawText( pcLogger.getVstOpCodeName( group, opCode ), pcOpCodeLedSize + 4, 0, pcOpCodeWidth, pcOpCodeHeight,
				Justification::centredLeft, true );

	// and count if not zero, try to fit it into place since the count can grow a lot
	if ( count > 0 )
		g.drawFittedText( String( count ), pcOpCodeWidth - pcOpCodeCounterWidth, 0,
											pcOpCodeCounterWidth - 2, pcOpCodeHeight,
											Justification::centredRight, 1 );
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeComponent::mouseDown( const MouseEvent &e )
{
	toggleEnabledState();
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeComponent::setEnabledState( bool state )
{
	pcLogger.setVstOpCodeEnabled( state, group, opCode );
	repaint();
}

//-----------------------------------------------------------------------------------------
void PCVstOpCodeComponent::toggleEnabledState()
{
	pcLogger.setVstOpCodeEnabled( !pcLogger.isVstOpCodeEnabled( group, opCode ), group, opCode );
	repaint();
}

//-----------------------------------------------------------------------------------------
Button* PCLookAndFeel::createDialogBoxCloseButton ()
{
	// after sized, add a minimize button next to the on top button
	minimizeButton = new ShapeButton( "PCMinimizeButton", pcTitleButtonNormalColor, pcTitleButtonOverColor,
															pcTitleButtonDownColor );
	minimizeButton->setTooltip( "Minimize" );
	Path path;
	path.addArrow( 0.5f, 0.1f, 0.5f, 0.35f, 0, 0.4f, 0.4f );
	minimizeButton->setShape( path, false, true, false );
	return minimizeButton;
}

//-----------------------------------------------------------------------------------------
PCWindow::PCWindow() : DialogWindow ("Plugin Consultant - " + pcLogger.getPlugName(), pcBackColor )
{
	// custom look and feel for putting the always on top button instead of close button
	setLookAndFeel( &pcLookAndFeel );

	// clear close (now ontop) button shortcuts since it they are not needed
	pcLookAndFeel.minimizeButton->clearShortcuts();
	
	// create content component and then size everything
	PCContentComponent* contentComp = new PCContentComponent();
	setContentComponent( contentComp );
	centreWithSize( pcWindowW, pcWindowH );

	// after sized, init content component
	contentComp->init();
	setVisible (true);
}

//-----------------------------------------------------------------------------------------
PCWindow::~PCWindow()
{
	deleteAllChildren();
}

//-----------------------------------------------------------------------------------------
void PCWindow::closeButtonPressed()
{
	// do nothing since we the host that will close us...
}

//-----------------------------------------------------------------------------------------
void PCWindow::userTriedToCloseWindow()
{
	// do nothing, window closed only if plugin is unloaded...
}

//-----------------------------------------------------------------------------------------
void PCWindow::timerCallback()
{
	((PCContentComponent*)getContentComponent())->updateGUI();
}

//-----------------------------------------------------------------------------------------
void  PCWindow::buttonClicked (Button* button)
{
	DialogWindow::buttonClicked( button );

	if ( button == pcLookAndFeel.minimizeButton )
		getNativeWindow()->setMinimised( true );
}
